@@ -82,7 +82,7 @@ import java.util.StringTokenizer;
  * @author Hein Meling
  * @author <a href="mailto:russel.winder@concertant.com">Russel Winder</a>
  * @author Danno Ferrin
- * @version $Revision: 19654 $
+ * @version $Revision: 20356 $
  */
 public class Groovyc extends MatchingTask {
     private final LoggingHelper log = new LoggingHelper(this);
@@ -100,6 +100,8 @@ public class Groovyc extends MatchingTask {
     private File forkJDK;
     private String memoryInitialSize;
     private String memoryMaximumSize;
+    private String scriptExtension = "*.groovy";
+    private String targetBytecode = null;
 
     protected boolean failOnError = true;
     protected boolean listFiles = false;
@@ -115,6 +117,8 @@ public class Groovyc extends MatchingTask {
     private boolean jointCompilation;
 
     private List<File> temporaryFiles = new ArrayList(2);
+    private File stubDir;
+    private boolean keepStubs;
 
 
     /**
@@ -161,6 +165,51 @@ public class Groovyc extends MatchingTask {
         return src;
     }
 
+    /**
+     * Set the extension to use when searching for Groovy source files. 
+     * Accepts extensions in the form *.groovy, .groovy or groovy
+     *
+     * @param scriptExtension the extension of Groovy source files
+     */
+    public void setScriptExtension(String scriptExtension) {
+    	if(scriptExtension.startsWith("*.")) {
+    		this.scriptExtension = scriptExtension;
+    	} else if(scriptExtension.startsWith(".")) {
+    		this.scriptExtension = "*" + scriptExtension;
+    	} else {
+    		this.scriptExtension = "*." + scriptExtension;
+    	}
+    }
+    
+    /**
+     * Get the extension to use when searching for Groovy source files.
+     *
+     * @return the extension of Groovy source files
+     */
+    public String getScriptExtension() {
+    	return scriptExtension;
+    }
+    
+    /**
+     * Sets the bytecode compatibility mode
+     * 
+     * @param version the bytecode compatibility mode
+     */
+    public void setTargetBytecode(String version) {
+        if(CompilerConfiguration.PRE_JDK5.equals(version) || CompilerConfiguration.POST_JDK5.equals(version)) {
+            this.targetBytecode = version;
+        }
+    }
+
+    /**
+     * Retrieves the compiler bytecode compatibility mode.
+     * 
+     * @return bytecode compatibility mode. Can be either <tt>1.5</tt> or <tt>1.4</tt>.
+     */
+    public String getTargetBytecode() {
+        return this.targetBytecode;
+    }
+    
     /**
      * Set the destination directory into which the Java source
      * files should be compiled.
@@ -572,7 +621,7 @@ public class Groovyc extends MatchingTask {
      */
     protected void scanDir(File srcDir, File destDir, String[] files) {
         GlobPatternMapper m = new GlobPatternMapper();
-        m.setFrom("*.groovy");
+        m.setFrom(scriptExtension);
         m.setTo("*.class");
         SourceFileScanner sfs = new SourceFileScanner(this);
         File[] newFiles = sfs.restrictAsFiles(files, srcDir, destDir, m);
@@ -667,7 +716,7 @@ public class Groovyc extends MatchingTask {
                                 || (key.contains("encoding"))
                                 || (key.contains("source"))
                                 || (key.contains("target"))
-                                || (key.contains("verbose"))) {
+                                || (key.contains("verbose"))) { // TODO remove extra verbose?
                             jointOptions.add("-J" + key + "=" + value);
                         } else {
                             log("The option " + key + " cannot be set on the contained <javac> element. The option will be ignored", Project.MSG_WARN);
@@ -721,6 +770,9 @@ public class Groovyc extends MatchingTask {
                     if ((fileEncodingProp != null) && !fileEncodingProp.equals("")) {
                         commandLineList.add("-Dfile.encoding=" + fileEncodingProp);
                     }
+                    if ((targetBytecode != null)) {
+                        commandLineList.add("-Dgroovy.target.bytecode=" + targetBytecode);
+                    }
 
                     if ((memoryInitialSize != null) && !memoryInitialSize.equals("")) {
                         commandLineList.add("-Xms" + memoryInitialSize);
@@ -824,6 +876,10 @@ public class Groovyc extends MatchingTask {
 
                         fileNameErrors = fileNameErrors && !FileSystemCompiler.validateFiles(filenames);
 
+                        if(targetBytecode != null) {
+                        	configuration.setTargetBytecode(targetBytecode);
+                        }
+                        
                         if (!fileNameErrors) {
                             FileSystemCompiler.doCompilation(configuration, makeCompileUnit(), filenames);
                         }
@@ -859,12 +915,18 @@ public class Groovyc extends MatchingTask {
     }
 
     protected CompilationUnit makeCompileUnit() {
-        if (configuration.getJointCompilationOptions() != null) {
-            if (!configuration.getJointCompilationOptions().containsKey("stubDir")) {
+        Map<String, Object> options = configuration.getJointCompilationOptions();
+        if (options != null) {
+            if (keepStubs) {
+                options.put("keepStubs", Boolean.TRUE);
+            }
+            if (stubDir != null) {
+                options.put("stubDir", stubDir);
+            } else {
                 try {
                     File tempStubDir = FileSystemCompiler.createTempDir();
                     temporaryFiles.add(tempStubDir);
-                    configuration.getJointCompilationOptions().put("stubDir", tempStubDir);
+                    options.put("stubDir", tempStubDir);
                 } catch (IOException ioe) {
                     throw new BuildException(ioe);
                 }
@@ -911,14 +973,15 @@ public class Groovyc extends MatchingTask {
 
     /**
      * Set the stub directory into which the Java source stub
-     * files should be generated. The directory should exist 
-     * will not be deleted automatically.
+     * files should be generated. The directory need not exist
+     * and will not be deleted automatically - though its contents
+     * will be cleared unless 'keepStubs' is true. Ignored when forked.
      *
      * @param stubDir the stub directory
      */
     public void setStubdir(File stubDir) {
         jointCompilation = true;
-        configuration.getJointCompilationOptions().put("stubDir", stubDir);
+        this.stubDir = stubDir;
     }
 
     /**
@@ -928,6 +991,25 @@ public class Groovyc extends MatchingTask {
      * @return the stub directory
      */
     public File getStubdir() {
-        return (File) configuration.getJointCompilationOptions().get("stubDir");
+        return stubDir;
+    }
+
+    /**
+     * Set the keepStubs flag. Defaults to false. Set to true for debugging.
+     * Ignored when forked.
+     *
+     * @param keepStubs should stubs be retained
+     */
+    public void setKeepStubs(boolean keepStubs) {
+        this.keepStubs = keepStubs;
+    }
+
+    /**
+     * Gets the keepStubs flag.
+     *
+     * @return the keepStubs flag
+     */
+    public boolean getKeepStubs() {
+        return keepStubs;
     }
 }
