@@ -21,12 +21,13 @@ import java.util.*;
  * Base class for any AST node which is capable of being annotated
  *
  * @author <a href="mailto:jstrachan@protique.com">James Strachan</a>
- * @version $Revision: 15807 $
+ * @version $Revision: 19865 $
  */
 public class AnnotatedNode extends ASTNode {
     private List<AnnotationNode> annotations = Collections.emptyList();
     private boolean synthetic;
     ClassNode declaringClass;
+    private boolean hasNoRealSourcePositionFlag;
 
     public AnnotatedNode() {
     }
@@ -92,4 +93,12 @@ public class AnnotatedNode extends ASTNode {
     public void setDeclaringClass(ClassNode declaringClass) {
         this.declaringClass = declaringClass;
     }
+
+    public boolean hasNoRealSourcePosition() {
+        return hasNoRealSourcePositionFlag;
+    }
+
+    public void setHasNoRealSourcePosition(boolean value) {
+        this.hasNoRealSourcePositionFlag = value;
+    }
 }
