@@ -17,13 +17,14 @@ package org.codehaus.groovy.ast.expr;
 
 import groovy.lang.MetaMethod;
 import org.codehaus.groovy.ast.ClassHelper;
+import org.codehaus.groovy.ast.GenericsType;
 import org.codehaus.groovy.ast.GroovyCodeVisitor;
 
 /**
  * A method call on an object or class
  * 
  * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
- * @version $Revision: 19268 $
+ * @version $Revision: 19974 $
  */
 public class MethodCallExpression extends Expression {
 
@@ -34,6 +35,10 @@ public class MethodCallExpression extends Expression {
     private boolean safe = false;
     private boolean implicitThis;
     
+    // type spec for generics
+    private GenericsType[] genericsTypes = null;
+    private boolean usesGenerics = false;
+    
     public static final Expression NO_ARGUMENTS = new TupleExpression();
 
     /**
@@ -170,4 +175,17 @@ public class MethodCallExpression extends Expression {
         //this.metaMethod = mmeth;
         //super.setType(ClassHelper.make(mmeth.getReturnType()));
     }
+
+    public GenericsType[] getGenericsTypes() {
+        return genericsTypes;
+    }
+
+    public void setGenericsTypes(GenericsType[] genericsTypes) {
+        usesGenerics = usesGenerics || genericsTypes != null;
+        this.genericsTypes = genericsTypes;
+    }
+
+    public boolean isUsingGenerics() {
+        return usesGenerics;
+    }
 }
