@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2007 the original author or authors.
+ * Copyright 2003-2010 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -13,7 +13,6 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.codehaus.groovy.tools.javac;
 
 import org.codehaus.groovy.ast.*;
@@ -22,6 +21,7 @@ import org.codehaus.groovy.ast.expr.ConstantExpression;
 import org.codehaus.groovy.ast.expr.ConstructorCallExpression;
 import org.codehaus.groovy.ast.expr.Expression;
 import org.codehaus.groovy.ast.expr.VariableExpression;
+import org.codehaus.groovy.ast.expr.PropertyExpression;
 import org.codehaus.groovy.ast.stmt.BlockStatement;
 import org.codehaus.groovy.ast.stmt.ExpressionStatement;
 import org.codehaus.groovy.ast.stmt.Statement;
@@ -30,8 +30,16 @@ import org.codehaus.groovy.control.ResolveVisitor;
 import org.codehaus.groovy.tools.Utilities;
 import org.objectweb.asm.Opcodes;
 
-import java.io.*;
-import java.util.*;
+import java.io.File;
+import java.io.FileNotFoundException;
+import java.io.FileOutputStream;
+import java.io.IOException;
+import java.io.PrintWriter;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Map;
 
 public class JavaStubGenerator
 {
@@ -41,6 +49,7 @@ public class JavaStubGenerator
     private List<String> toCompile = new ArrayList<String>();
     private ArrayList<MethodNode> propertyMethods = new ArrayList<MethodNode>();
     private ArrayList<ConstructorNode> constructors = new ArrayList<ConstructorNode>();
+    private ModuleNode currentModule;
 
     public JavaStubGenerator(final File outputPath, final boolean requireSuperResolved, final boolean java5) {
         this.outputPath = outputPath;
@@ -106,10 +115,10 @@ public class JavaStubGenerator
     }
 
     private void genClassInner(ClassNode classNode, PrintWriter out) throws FileNotFoundException {
-    	if(classNode instanceof InnerClassNode && ((InnerClassNode) classNode).isAnonymous()) {
-    		// if it is an anonymous inner class, don't generate the stub code for it.
-    		return;
-    	}
+        if (classNode instanceof InnerClassNode && ((InnerClassNode) classNode).isAnonymous()) {
+            // if it is an anonymous inner class, don't generate the stub code for it.
+            return;
+        }
         try {
             Verifier verifier = new Verifier() {
                 public void addCovariantMethods(ClassNode cn) {}
@@ -140,9 +149,11 @@ public class JavaStubGenerator
                 }
             };
             verifier.visitClass(classNode);
+            currentModule = classNode.getModule();
 
             boolean isInterface = classNode.isInterface();
             boolean isEnum = (classNode.getModifiers() & Opcodes.ACC_ENUM) !=0;
+            printAnnotations(out, classNode);
             printModifiers(out, classNode.getModifiers()
                     & ~(isInterface ? Opcodes.ACC_ABSTRACT : 0));
 
@@ -184,14 +195,14 @@ public class JavaStubGenerator
             }
             out.println(" {");
 
-            genFields(classNode, out, isEnum);
+            genFields(classNode, out);
             genMethods(classNode, out, isEnum);
 
-            for (Iterator<InnerClassNode> inner = classNode.getInnerClasses(); inner.hasNext(); ) {
-            	// GROOVY-4004: Clear the property methods from the outer class so that they don't get duplicated in inner ones
-            	propertyMethods.clear();
+            for (Iterator<InnerClassNode> inner = classNode.getInnerClasses(); inner.hasNext();) {
+                // GROOVY-4004: Clear the methods from the outer class so that they don't get duplicated in inner ones
+                propertyMethods.clear();
                 constructors.clear();
-            	genClassInner(inner.next(), out);
+                genClassInner(inner.next(), out);
             }
 
             out.println("}");
@@ -199,89 +210,97 @@ public class JavaStubGenerator
         finally {
             propertyMethods.clear();
             constructors.clear();
+            currentModule = null;
         }
     }
 
     private void genMethods(ClassNode classNode, PrintWriter out, boolean isEnum) {
         if (!isEnum) getConstructors(classNode, out);
 
-        List methods = (List) propertyMethods.clone();
+        @SuppressWarnings("unchecked")
+        List<MethodNode> methods = (List)propertyMethods.clone();
         methods.addAll(classNode.getMethods());
-        if (methods != null)
-            for (Iterator it = methods.iterator(); it.hasNext();) {
-                MethodNode methodNode = (MethodNode) it.next();
-                if(isEnum && methodNode.isSynthetic()) {
-                    // skip values() method and valueOf(String)
-                    String name = methodNode.getName();
-                    Parameter[] params = methodNode.getParameters();
-                    if (name.equals("values") && params.length==0) continue;
-                    if (name.equals("valueOf") &&
-                        params.length==1 &&
-                        params[0].getType().equals(ClassHelper.STRING_TYPE))
-                    {
-                        continue;
-                    }
+        for (MethodNode method : methods) {
+            if (isEnum && method.isSynthetic()) {
+                // skip values() method and valueOf(String)
+                String name = method.getName();
+                Parameter[] params = method.getParameters();
+                if (name.equals("values") && params.length == 0) continue;
+                if (name.equals("valueOf") &&
+                        params.length == 1 &&
+                        params[0].getType().equals(ClassHelper.STRING_TYPE)) {
+                    continue;
                 }
-                genMethod(classNode, methodNode, out);
             }
+            genMethod(classNode, method, out);
+        }
     }
 
     private void getConstructors(ClassNode classNode, PrintWriter out) {
         List<ConstructorNode> constrs = (List<ConstructorNode>) constructors.clone();
         constrs.addAll(classNode.getDeclaredConstructors());
         if (constrs != null)
-            for (Iterator it = constrs.iterator(); it.hasNext();) {
-                ConstructorNode constrNode = (ConstructorNode) it.next();
-                genConstructor(classNode, constrNode, out);
+            for (ConstructorNode constr : constrs) {
+                genConstructor(classNode, constr, out);
             }
     }
 
-    private void genFields(ClassNode classNode, PrintWriter out, boolean isEnum) {
-        List fields = classNode.getFields();
+    private void genFields(ClassNode classNode, PrintWriter out) {
+        boolean isInterface = classNode.isInterface();
+        List<FieldNode> fields = classNode.getFields();
         if (fields == null) return;
-        ArrayList enumFields = new ArrayList(fields.size());
-        ArrayList normalFields = new ArrayList(fields.size());
-        for (Iterator it = fields.iterator(); it.hasNext();) {
-            FieldNode fieldNode = (FieldNode) it.next();
-            boolean isEnumField = (fieldNode.getModifiers() & Opcodes.ACC_ENUM) !=0;
-            boolean isSynthetic = (fieldNode.getModifiers() & Opcodes.ACC_SYNTHETIC) !=0;
-            if (isEnumField) {
-                enumFields.add(fieldNode);
+        List<FieldNode> enumFields = new ArrayList<FieldNode>(fields.size());
+        List<FieldNode> normalFields = new ArrayList<FieldNode>(fields.size());
+        for (FieldNode field : fields) {
+            boolean isSynthetic = (field.getModifiers() & Opcodes.ACC_SYNTHETIC) != 0;
+            if (field.isEnum()) {
+                enumFields.add(field);
             } else if (!isSynthetic) {
-                normalFields.add(fieldNode);
+                normalFields.add(field);
             }
         }
         genEnumFields(enumFields, out);
-        for (Iterator iterator = normalFields.iterator(); iterator.hasNext();) {
-            FieldNode fieldNode = (FieldNode) iterator.next();
-            genField(fieldNode, out);
+        for (FieldNode normalField : normalFields) {
+            genField(normalField, out, isInterface);
         }
     }
 
 
-    private void genEnumFields(List fields, PrintWriter out) {
+    private void genEnumFields(List<FieldNode> fields, PrintWriter out) {
         if (fields.size()==0) return;
         boolean first = true;
-        for (Iterator iterator = fields.iterator(); iterator.hasNext();) {
-            FieldNode fieldNode = (FieldNode) iterator.next();
+        for (FieldNode field : fields) {
             if (!first) {
                 out.print(", ");
             } else {
                 first = false;
             }
-            out.print(fieldNode.getName());
+            out.print(field.getName());
         }
         out.println(";");
     }
 
-    private void genField(FieldNode fieldNode, PrintWriter out) {
+    private void genField(FieldNode fieldNode, PrintWriter out, boolean isInterface) {
         if ((fieldNode.getModifiers()&Opcodes.ACC_PRIVATE)!=0) return;
-        printModifiers(out, fieldNode.getModifiers());
+        printAnnotations(out, fieldNode);
+        if (!isInterface) {
+            printModifiers(out, fieldNode.getModifiers());
+        }
 
-        printType(fieldNode.getType(), out);
+        ClassNode type = fieldNode.getType();
+        printType(type, out);
 
         out.print(" ");
         out.print(fieldNode.getName());
+        if (isInterface) {
+            out.print(" = ");
+            if (ClassHelper.isPrimitiveType(type)) {
+                String val = type == ClassHelper.boolean_TYPE ? "false" : "0";
+                out.print("new " + ClassHelper.getWrapper(type) + "((" + type + ")" + val + ")");
+            } else {
+                out.print("null");
+            }
+        }
         out.println(";");
     }
 
@@ -308,6 +327,7 @@ public class JavaStubGenerator
     }
 
     private void genConstructor(ClassNode clazz, ConstructorNode constructorNode, PrintWriter out) {
+        printAnnotations(out, constructorNode);
         // printModifiers(out, constructorNode.getModifiers());
 
         out.print("public "); // temporary hack
@@ -332,10 +352,7 @@ public class JavaStubGenerator
         ClassNode type = node.getDeclaringClass();
         ClassNode superType = type.getSuperClass();
 
-        boolean hadPrivateConstructor = false;
-        for (Iterator iter = superType.getDeclaredConstructors().iterator(); iter.hasNext();) {
-            ConstructorNode c = (ConstructorNode)iter.next();
-
+        for (ConstructorNode c : superType.getDeclaredConstructors()) {
             // Only look at things we can actually call
             if (c.isPublic() || c.isProtected()) {
                 return c.getParameters();
@@ -352,7 +369,7 @@ public class JavaStubGenerator
 
     private void genSpecialConstructorArgs(PrintWriter out, ConstructorNode node, ConstructorCallExpression constrCall) {
         // Select a constructor from our class, or super-class which is legal to call,
-        // then write out an invoke w/nulls using casts to avoid abigous crapo
+        // then write out an invoke w/nulls using casts to avoid ambiguous crapo
 
         Parameter[] params = selectAccessibleConstructorFromSuper(node);
         if (params != null) {
@@ -382,11 +399,9 @@ public class JavaStubGenerator
         // Else try to render some arguments
         if (arguments instanceof ArgumentListExpression) {
             ArgumentListExpression argumentListExpression = (ArgumentListExpression) arguments;
-            List args = argumentListExpression.getExpressions();
-
-            for (Iterator it = args.iterator(); it.hasNext();) {
-                Expression arg = (Expression) it.next();
+            List<Expression> args = argumentListExpression.getExpressions();
 
+            for (Expression arg : args) {
                 if (arg instanceof ConstantExpression) {
                     ConstantExpression expression = (ConstantExpression) arg;
                     Object o = expression.getValue();
@@ -414,10 +429,9 @@ public class JavaStubGenerator
         if (!(arg instanceof VariableExpression)) return arg.getType();
         VariableExpression vexp = (VariableExpression) arg;
         String name = vexp.getName();
-        Parameter[] params = node.getParameters();
-        for (int i=0; i<params.length; i++) {
-            if (params[i].getName().equals(name)) {
-                return params[i].getType();
+        for (Parameter param : node.getParameters()) {
+            if (param.getName().equals(name)) {
+                return param.getType();
             }
         }
         return vexp.getType();
@@ -427,6 +441,7 @@ public class JavaStubGenerator
         if (methodNode.getName().equals("<clinit>")) return;
         if (methodNode.isPrivate() || !Utilities.isJavaIdentifier(methodNode.getName())) return;
 
+        printAnnotations(out, methodNode);
         if (!clazz.isInterface()) printModifiers(out, methodNode.getModifiers());
 
         writeGenericsBounds(out, methodNode.getGenericsTypes());
@@ -492,7 +507,7 @@ public class JavaStubGenerator
             printType(type.getComponentType(),out);
             out.print("[]");
         } else if (java5 && type.isGenericsPlaceHolder()) {
-        	out.print(type.getGenericsTypes()[0].getName());
+            out.print(type.getGenericsTypes()[0].getName());
         } else {
             writeGenericsBounds(out,type,false);
         }
@@ -520,7 +535,11 @@ public class JavaStubGenerator
                 out.print("void");
             }
         } else {
-            out.print(type.redirect().getName().replace('$', '.'));
+            String name = type.getName();
+            // check for an alias
+            ClassNode alias = currentModule.getImportType(name);
+            if (alias != null) name = alias.getName();
+            out.print(name.replace('$', '.'));
         }
     }
 
@@ -584,6 +603,37 @@ public class JavaStubGenerator
         out.print(")");
     }
 
+    private void printAnnotations(PrintWriter out, AnnotatedNode annotated) {
+        if (!java5) return;
+        for (AnnotationNode annotation : annotated.getAnnotations()) {
+            out.print("@" + annotation.getClassNode().getName() + "(");
+            boolean first = true;
+            Map<String, Expression> members = annotation.getMembers();
+            for (String key : members.keySet()) {
+                String val = "null";
+                Object memberValue = members.get(key);
+                if (memberValue instanceof ConstantExpression) {
+                    ConstantExpression ce = (ConstantExpression) memberValue;
+                    Object constValue = ce.getValue();
+                    if (constValue instanceof Number || constValue instanceof Boolean)
+                        val = constValue.toString();
+                    else
+                        val = "\"" + constValue + "\"";
+                } else if (memberValue instanceof PropertyExpression || memberValue instanceof VariableExpression) {
+                    // assume must be static class field or enum value or class that Java can resolve
+                    val = ((Expression) memberValue).getText();
+                }
+                if (first) {
+                    first = false;
+                } else {
+                    out.print(", ");
+                }
+                out.print(key + "=" + val);
+            }
+            out.print(") ");
+        }
+    }
+
     private void printModifiers(PrintWriter out, int modifiers) {
         if ((modifiers & Opcodes.ACC_PUBLIC) != 0)
             out.print("public ");
@@ -605,12 +655,7 @@ public class JavaStubGenerator
     }
 
     private void genImports(ClassNode classNode, PrintWriter out) {
-        Set<String> imports = new HashSet<String>();
-
-        //
-        // HACK: Add the default imports... since things like Closure and GroovyObject seem to parse out w/o fully qualified classnames.
-        //
-        imports.addAll(Arrays.asList(ResolveVisitor.DEFAULT_IMPORTS));
+        List<String> imports = new ArrayList<String>();
 
         ModuleNode moduleNode = classNode.getModule();
         for (ImportNode importNode : moduleNode.getStarImports()) {
@@ -618,23 +663,27 @@ public class JavaStubGenerator
         }
 
         for (ImportNode imp : moduleNode.getImports()) {
-            String name = imp.getType().getName();
-            int lastDot = name.lastIndexOf('.');
-            if (lastDot != -1)
-                imports.add(name.substring(0, lastDot + 1));
+            imports.add(imp.getType().getName());
         }
 
+        // TODO: find out why everything isn't fully resolved by now then delete next line
+        imports.addAll(Arrays.asList(ResolveVisitor.DEFAULT_IMPORTS));
+
         for (String imp : imports) {
-            out.print("import ");
-            out.print(imp);
-            out.println("*;");
+            String s = new StringBuilder()
+                .append("import ")
+                .append(imp)
+                .append((imp.charAt(imp.length() - 1) == '.') ? "*;" : ";")
+                .toString()
+                .replace('$', '.');
+
+            out.println(s);
         }
         out.println();
     }
 
     public void clean() {
-        for (Iterator it = toCompile.iterator(); it.hasNext();) {
-            String path = (String) it.next();
+        for (String path : toCompile) {
             new File(outputPath, path + ".java").delete();
         }
     }
